// NetUINotification.cpp : Defines the entry point for the console application.
//

#include "windows.h"

#define NETUIPOLICY_NOTIFICATION_QUEUE         _T("NetUI/NotificationQueue")

EXTERN_C DWORD WINAPI  NetUIMonitorThreadProc(LPVOID lpvParam);

static HANDLE s_hMsgQ;
static HANDLE s_hExit;

// This routine creates a message queue for receiving notifications from appliations
// and other drivers.  It returns a handle to the queue, or NULL.
HANDLE
NetUICreateNotificationQueue(void)
{
     MSGQUEUEOPTIONS opts = {
        sizeof(opts),
        0,
        20,
        MAX_PATH*2,
        TRUE
    };


    // create the queue
    s_hMsgQ = CreateMsgQueue(NETUIPOLICY_NOTIFICATION_QUEUE, &opts);
    if(s_hMsgQ == NULL) {
        NKDbgPrintfW(_T("CreateMsgQueue() failed %d\r\n"),GetLastError());
    }
    //GetMsgQueue(
    return s_hMsgQ;
}

// This routine closes a message queue opened with CreateNotificationQueue().
void
NetUICloseNotificationQueue()
{
    CloseMsgQueue(s_hMsgQ);
}

// This routine reads a message from the notification queue.  It returns ERROR_SUCCESS
// on completion when the caller's buffer has been filled in or a Win32 error code
// if there's a problem.
DWORD
NetUIReadNotificationQueue(HANDLE hqNotify, LPVOID pvBuffer, DWORD dwSize)
{
    DWORD dwBytesRead, dwFlags, dwStatus;

    // determine what's happened
    BOOL fOk = ReadMsgQueue(hqNotify, pvBuffer, dwSize, &dwBytesRead, 0, &dwFlags);
    if(!fOk) {
        dwStatus = GetLastError();
        // msg queue will get signalled on a process exit also. In this case
        // ReadMsgQueue will be empty and fail with ERROR_TIMEOUT
        if (dwStatus != ERROR_TIMEOUT) {
            NKDbgPrintfW(_T("ReadMsgQueue() failed %d\r\n"),dwStatus);
        }
    } 
	else {
		
		NKDbgPrintfW(_T(" %s %d \r\n"),(TCHAR *)pvBuffer, dwBytesRead);
		printf("\r\nTID 0x%08x %S", GetCurrentThreadId(), (TCHAR *)pvBuffer);
		
        dwStatus = ERROR_SUCCESS;
    }

    return dwStatus;
}

int _tmain(int argc, _TCHAR* argv[])
{
	
	printf("\r\n NetUI Notification monitor Started!!!");

    s_hExit = CreateEvent(NULL, FALSE, FALSE, NULL);

    HANDLE htMonitor = CreateThread(NULL, 0, NetUIMonitorThreadProc, NULL, 0, NULL);
    if(htMonitor == NULL) {
        NKDbgPrintfW(_T("CreateThread() failed %d for power monitor thread\r\n"),GetLastError());
    } else {
    CloseHandle(htMonitor);
    }

    getchar();

    SetEvent(s_hExit);
    WaitForSingleObject(htMonitor, 30000);
    CloseHandle(htMonitor);
    return 0;
}

// This thread monitors netui notification and generates update messages as appropriate.
EXTERN_C DWORD WINAPI NetUIMonitorThreadProc(LPVOID lpvParam)
{
    BOOL fDone = FALSE;
    DWORD dwRead;
    TCHAR rgInputs[MAX_PATH];

    UNREFERENCED_PARAMETER(lpvParam);

    NKDbgPrintfW(_T("TID 0x%08x\r\n"),GetCurrentThreadId());
	
    // create our notification queue
    NetUICreateNotificationQueue();
    if(s_hMsgQ == NULL) {
        NKDbgPrintfW(_T("PmPolicyCreateNotificationQueue() failed\r\n"));
    }

     // Empty the queue to avoid replaying old events
    while (NetUIReadNotificationQueue(s_hMsgQ, rgInputs, sizeof(rgInputs)) == ERROR_SUCCESS)
		;			

    HANDLE rghWaits[] = { s_hMsgQ, s_hExit };

    while ( WAIT_OBJECT_0 == WaitForMultipleObjects(_countof(rghWaits), rghWaits, FALSE, INFINITE) )
    {
        NetUIReadNotificationQueue(s_hMsgQ, rgInputs, sizeof(rgInputs));
        Sleep(10);        
    }

    NetUICloseNotificationQueue();

    return 0;
}

